package com.tzld.piaoquan.wecom.service.Impl;

import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.wecom.component.HttpPoolClient;
import com.tzld.piaoquan.wecom.dao.mapper.SubjectMapper;
import com.tzld.piaoquan.wecom.model.po.Subject;
import com.tzld.piaoquan.wecom.model.po.SubjectExample;
import com.tzld.piaoquan.wecom.service.AccessTokenService;
import com.tzld.piaoquan.wecom.utils.HttpClientUtil;
import com.tzld.piaoquan.wecom.utils.LarkRobotUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.List;
import java.util.concurrent.TimeUnit;

import static com.tzld.piaoquan.wecom.common.constant.RedisConstant.ACCESS_TOKEN;
import static com.tzld.piaoquan.wecom.common.constant.RedisConstant.WE_COM_ACCESS_TOKEN;
import static com.tzld.piaoquan.wecom.common.constant.WeComConstant.*;

@Slf4j
@Service
public class AccessTokenServiceImpl implements AccessTokenService {

    @Autowired
    private HttpPoolClient httpPoolClient;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private SubjectMapper subjectMapper;

    @Override
    public String getWeComAccessToken(Long subjectId) {
        try {
            String key = String.format(WE_COM_ACCESS_TOKEN, subjectId);
            String accessToken = (String) redisTemplate.opsForValue().get(key);
            if (StringUtils.isNotEmpty(accessToken)) {
                return accessToken;
            }
            SubjectExample example = new SubjectExample();
            example.createCriteria().andIdEqualTo(subjectId);
            List<Subject> subjects = subjectMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(subjects)) {
                LarkRobotUtil.sendMessage("查询不到主体，获取access_token失败");
                return "";
            }
            Subject subject = subjects.get(0);
            String cropId = subject.getCropId();
            String secret = subject.getSecret();
            String res = httpPoolClient.get(String.format(GET_WE_COM_ACCESS_TOKEN_URL +
                    "?corpid=%s&corpsecret=%s", cropId, secret));
            JSONObject jsonObject = JSONObject.parseObject(res);
            Long expiresIn = jsonObject.getLong("expires_in");
            String newAccessToken = jsonObject.getString("access_token");
            redisTemplate.opsForValue().set(key, newAccessToken, expiresIn, TimeUnit.SECONDS);
            return newAccessToken;
        } catch (IOException e) {
            LarkRobotUtil.sendMessage("getWeComAccessToken error:" + e);
            log.error("getWeComAccessToken error", e);
        }
        return "";
    }
}
