package com.tzld.piaoquan.wecom.service.Impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.wecom.common.base.CommonResponse;
import com.tzld.piaoquan.wecom.common.enums.MessageAttachmentTypeEnum;
import com.tzld.piaoquan.wecom.component.HttpPoolClient;
import com.tzld.piaoquan.wecom.dao.mapper.MessageAttachmentMapper;
import com.tzld.piaoquan.wecom.model.bo.AdPutFlowParam;
import com.tzld.piaoquan.wecom.model.bo.VideoDetail;
import com.tzld.piaoquan.wecom.model.bo.VideoParam;
import com.tzld.piaoquan.wecom.model.po.MessageAttachment;
import com.tzld.piaoquan.wecom.model.po.MessageAttachmentExample;
import com.tzld.piaoquan.wecom.model.po.Staff;
import com.tzld.piaoquan.wecom.model.vo.GuaranteedParam;
import com.tzld.piaoquan.wecom.service.AccessTokenService;
import com.tzld.piaoquan.wecom.service.MessageAttachmentService;
import com.tzld.piaoquan.wecom.utils.DateUtil;
import com.tzld.piaoquan.wecom.utils.LarkRobotUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.*;
import java.util.concurrent.TimeUnit;

import static com.tzld.piaoquan.wecom.common.constant.MessageConstant.MAX_VIDEO_NUM;
import static com.tzld.piaoquan.wecom.common.constant.MessageConstant.appid;
import static com.tzld.piaoquan.wecom.common.constant.OtherServerURL.POST_ADD_TENCENT;
import static com.tzld.piaoquan.wecom.common.constant.OtherServerURL.POST_VIDEO_DETAIL_URL;
import static com.tzld.piaoquan.wecom.common.constant.RedisConstant.GUARANTEED_MINI_PROGRAM_KEY;
import static com.tzld.piaoquan.wecom.common.constant.WeComConstant.POST_WE_COM_MEDIA_UPLOAD;


@Slf4j
@Service
public class MessageAttachmentServiceImpl implements MessageAttachmentService {

    @Autowired
    private HttpPoolClient httpPoolClient;
    @Autowired
    private AccessTokenService accessTokenService;

    @Autowired
    private MessageAttachmentMapper messageAttachmentMapper;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Override
    public void addMiniProgram(List<MessageAttachment> messageAttachmentList) {
        for (MessageAttachment messageAttachment : messageAttachmentList) {
            MessageAttachmentExample example = new MessageAttachmentExample();
            example.createCriteria()
                    .andAppidEqualTo(messageAttachment.getAppid())
                    .andTypeEqualTo(messageAttachment.getType())
                    .andMiniprogramVideoIdEqualTo(messageAttachment.getMiniprogramVideoId());
            List<MessageAttachment> messageAttachments = messageAttachmentMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(messageAttachments)) {
                messageAttachmentMapper.insert(messageAttachment);
                continue;
            }
            messageAttachment.setId(messageAttachments.get(0).getId());
            messageAttachmentMapper.updateByPrimaryKeySelective(messageAttachment);
        }
    }

    @Override
    public CommonResponse<Void> createGuaranteedMiniProgram(GuaranteedParam guaranteedParam) {
        if (guaranteedParam == null
                || StringUtils.isEmpty(guaranteedParam.getDate())
                || CollectionUtils.isEmpty(guaranteedParam.getVideoParamList())) {
            return CommonResponse.create(500, "参数错误");
        }
        List<Long> videoIds = new ArrayList<>();
        for (VideoParam videoParam : guaranteedParam.getVideoParamList()) {
            if (CollectionUtils.isEmpty(videoParam.getVideoIds()) || videoParam.getVideoIds().size() < MAX_VIDEO_NUM) {
                LarkRobotUtil.sendMessage("保底视频数量异常，请查看" + guaranteedParam.getDate());
            }
            videoIds.addAll(videoParam.getVideoIds());
        }
        Map<Long, VideoDetail> coverMap = getVideoDetail(videoIds);
        List<MessageAttachment> messageAttachmentList = new ArrayList<>();
        for (Long videoId : videoIds) {
            MessageAttachment messageAttachment = new MessageAttachment();
            VideoDetail videoDetail = coverMap.get(videoId);
            if (videoDetail == null || StringUtils.isEmpty(videoDetail.getCover()) || StringUtils.isEmpty(videoDetail.getTitle())) {
                LarkRobotUtil.sendMessage("获取视频详情异常，请查看" + videoId);
                throw new RuntimeException("获取视频详情异常");
            }
            messageAttachment.setMiniprogramVideoId(videoId);
            messageAttachment.setType(MessageAttachmentTypeEnum.MINI_PROGRAM.getType());
            messageAttachment.setCover(videoDetail.getCover());
            messageAttachment.setTitle(videoDetail.getTitle());
            messageAttachment.setAppid(appid);
            messageAttachmentList.add(messageAttachment);
        }
        addMiniProgram(messageAttachmentList);
        String date = guaranteedParam.getDate();
        String key = String.format(GUARANTEED_MINI_PROGRAM_KEY, date);
        redisTemplate.opsForValue().set(key, guaranteedParam);
        return CommonResponse.success();
    }

    public Map<Long, VideoDetail> getVideoDetail(List<Long> videoIdList) {
        try {
            Map<Long, VideoDetail> map = new HashMap<>();
            JSONObject params = new JSONObject();
            params.put("videoIdList", videoIdList);
            String post = httpPoolClient.post(POST_VIDEO_DETAIL_URL, params.toJSONString());
            JSONObject res = JSONObject.parseObject(post);
            JSONArray data = res.getJSONArray("data");
            for (int i = 0; i < data.size(); i++) {
                VideoDetail videoDetail = new VideoDetail();
                JSONObject jsonObject = data.getJSONObject(i);
                Long videoId = jsonObject.getLong("id");
                String shareImgPath = jsonObject.getString("shareImgPath");
                String title = jsonObject.getString("title");
                videoDetail.setCover(shareImgPath);
                videoDetail.setTitle(title);
                map.put(videoId, videoDetail);
            }
            return map;
        } catch (IOException e) {
            log.error("MessageAttachmentServiceImpl getVideDetail error", e);
        }
        return new HashMap<>();
    }

    @Override
    public String getPicMediaId(String cover, Long subjectId) {
        String mediaId = (String) redisTemplate.opsForValue().get(cover);
        if (StringUtils.isNotEmpty(mediaId)) {
            return mediaId;
        }
        String filePath = UUID.randomUUID() + ".jpg"; // 临时文件路径
        try {
            HttpURLConnection httpUrl = (HttpURLConnection) new URL(cover).openConnection();
            httpUrl.connect();
            InputStream inputStream = httpUrl.getInputStream();

            // 将文件内容写入临时文件
            try (OutputStream outputStream = Files.newOutputStream(Paths.get(filePath))) {
                byte[] buffer = new byte[4096];
                int bytesRead;
                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                }
            }
            inputStream.close();
            httpUrl.disconnect();
            File file = new java.io.File(filePath);
            String weComAccessToken = accessTokenService.getWeComAccessToken(subjectId);
            String url = String.format(POST_WE_COM_MEDIA_UPLOAD + "?access_token=%s&type=%s", weComAccessToken, "image");
            String res = httpPoolClient.post(url, file);
            JSONObject jsonObject = JSONObject.parseObject(res);
            if (jsonObject != null && jsonObject.getInteger("errcode") == 0) {
                mediaId = jsonObject.getString("media_id");
                redisTemplate.opsForValue().set(cover, mediaId, 2, TimeUnit.DAYS);
            }
            Files.delete(Paths.get(filePath));
        } catch (Exception e) {
            log.error("getPicMediaId error", e);
        }
        return mediaId;
    }


    @Override
    public String getPage(Staff staff, Long videoId) {
        try {
            if (staff == null || staff.getCarrierId() == null) {
                return "";
            }
            String page = selectPage(videoId, staff.getCarrierId(), "企微", "日常推送");
            if (StringUtils.isNotEmpty(page)) {
                return page;
            }
            return insertPage(staff.getCarrierId(), videoId, staff.getRemark());
        } catch (IOException e) {
            log.error("MessageAttachmentService getPage error", e);
        }
        return null;
    }

    private String insertPage(String carrierId, Long videoId, String remark) throws IOException {
        AdPutFlowParam param = new AdPutFlowParam();
        param.setChannel("tencentqw");
        param.setPath("pages/category.html");
        param.setPutCarrierId(carrierId);
        param.setPutScene("touliu");
        param.setPutStartDate(DateUtil.getDayDateString("yyyy-MM-dd"));
        param.setPutTypeOne("企微");
        param.setPutTypeTwo("日常推送");
        param.setPutTypeThree("位置X");
        param.setRemark(remark);
        param.setVideoId(videoId);
        Map<String, String> requestParam = new HashMap<>();
        String jumpPage = "pages/user-videos?id=${videoId}&fromGzh=1&rootShareId=${uuid}&shareId=${uuid}&rootSourceId=[rootSourceId]"
                .replace("${videoId}", "" + videoId)
                .replace("${uuid}", "" + UUID.randomUUID());
        requestParam.put("jumpPage", jumpPage);
        param.setRequestParam(requestParam);
        String res = httpPoolClient.post(POST_ADD_TENCENT, JSONObject.toJSONString(param));
        JSONObject jsonObject = JSONObject.parseObject(res);
        JSONObject data = jsonObject.getJSONObject("data");
        return data.getString("url");
    }

    public String selectPage(Long videoId, String carrierId, String typeOne, String typeTwo) {
        int totalPage = 1;
        for (int n = 1; n <= totalPage; n++) {
            String url = getUrl(videoId, n);
            try {
                String res = httpPoolClient.get(url);
                JSONObject jsonObject = JSONObject.parseObject(res);
                JSONObject data = jsonObject.getJSONObject("data");
                if (data == null) {
                    continue;
                }
                if (data.getInteger("totalPage") != null) {
                    totalPage = data.getInteger("totalPage");
                }
                JSONArray objs = data.getJSONArray("objs");
                for (int i = 0; i < objs.size(); i++) {
                    JSONObject obj = objs.getJSONObject(i);
                    String putCarrierId = obj.getString("putCarrierId");
                    String page = obj.getString("url");
                    String putTypeOne = obj.getString("putTypeOne");
                    String putTypeTwo = obj.getString("putTypeTwo");

                    if (StringUtils.isEmpty(putCarrierId) || StringUtils.isEmpty(page) || StringUtils.isEmpty(putTypeOne) || StringUtils.isEmpty(putTypeTwo)) {
                        continue;
                    }

                    if (putCarrierId.equals(carrierId) && putTypeOne.equals(typeOne) && putTypeTwo.equals(typeTwo)) {
                        return page;
                    }
                }
            } catch (IOException e) {
                log.error("Error fetching data from API for videoId: " + videoId + ", page: " + n, e);
            }
        }
        return null;
    }

    private static String getUrl(Long videoId, int n) {
        String baseUrl = "https://api.piaoquantv.com/ad/put/flow/list/tencent";
        String putScene = "touliu";
        String channel = "tencentqw";
        int pageSize = 20;
        return String.format(
                "%s?videoId=%d&putScene=%s&channel=%s&currentPage=%d&pageSize=%d",
                baseUrl,
                videoId,
                putScene,
                channel,
                n,
                pageSize
        );
    }
}
