package com.tzld.piaoquan.wecom.service.Impl;

import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.wecom.common.constant.MessageConstant;
import com.tzld.piaoquan.wecom.common.exception.CustomizeException;
import com.tzld.piaoquan.wecom.component.HttpPoolClient;
import com.tzld.piaoquan.wecom.model.vo.MessageTextParam;
import com.tzld.piaoquan.wecom.service.AccessTokenService;
import com.tzld.piaoquan.wecom.service.MessageService;
import com.tzld.piaoquan.wecom.utils.HttpClientUtil;
import com.tzld.piaoquan.wecom.utils.LarkRobotUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.concurrent.TimeUnit;

import static com.tzld.piaoquan.wecom.common.constant.RedisConstant.PUSH_MESSAGE_TEXT;
import static com.tzld.piaoquan.wecom.common.constant.WeComConstant.POST_WE_COM_ADD_MSG_TEMPLATE;
import static com.tzld.piaoquan.wecom.common.enums.ExceptionCodeEnum.PARAMS_ERROR;
import static com.tzld.piaoquan.wecom.common.constant.WeComConstant.POST_MESSAGE_PUSH_URL;


@Slf4j
@Service
public class MessageServiceImpl implements MessageService {

    @Autowired
    private HttpPoolClient httpPoolClient;

    @Autowired
    private AccessTokenService accessTokenService;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Override
    public boolean pushWeComMessage(JSONObject jsonObject, Long subjectId) {
        try {
            String accessToken = accessTokenService.getWeComAccessToken(subjectId);
            String url = POST_WE_COM_ADD_MSG_TEMPLATE
                    + "?access_token=" + accessToken;
            String s = httpPoolClient.post(url, jsonObject.toJSONString());
            JSONObject res = JSONObject.parseObject(s);
            log.info("pushWeComMessage res={}", res);
            Integer code = res.getInteger("errcode");
            if (code != 0) {
                LarkRobotUtil.sendMessage("pushWeComMessage error" + "res:" + res.toJSONString() + "jsonObject:" + jsonObject.toJSONString());
            }
            return code == 0;
        } catch (IOException e) {
            log.error("pushWeComMessage error", e);
            LarkRobotUtil.sendMessage("pushWeComMessage error" + "jsonObject:" + jsonObject.toJSONString() + "Exception:" + e);
        }
        return false;
    }

    @Override
    public void createMessageText(MessageTextParam messageTextParam) {
        if (messageTextParam == null || messageTextParam.getHour() == null) {
            throw new CustomizeException(PARAMS_ERROR);
        }
        if (messageTextParam.getHour() == -1) {
            redisTemplate.opsForValue().set(PUSH_MESSAGE_TEXT, messageTextParam.getText());
        } else {
            redisTemplate.opsForValue().set(PUSH_MESSAGE_TEXT, messageTextParam.getText(), messageTextParam.getHour(), TimeUnit.HOURS);
        }
    }


    @Override
    public String getMessageText() {
        String text = (String) redisTemplate.opsForValue().get(PUSH_MESSAGE_TEXT);
        if (StringUtils.isEmpty(text)) {
            return MessageConstant.guaranteedText;
        }
        return text;
    }
}
